@@ -1,5 +1,4 @@
 //go:build !remote
-// +build !remote
 
 package system
 
@@ -8,11 +7,9 @@ import (
 	"os"
 
 	"github.com/containers/common/pkg/completion"
-	"github.com/containers/podman/v4/cmd/podman/registry"
-	"github.com/containers/podman/v4/cmd/podman/validate"
-	"github.com/containers/podman/v4/libpod/define"
-	"github.com/containers/podman/v4/pkg/domain/entities"
-	"github.com/containers/podman/v4/pkg/domain/infra"
+	"github.com/containers/podman/v5/cmd/podman/registry"
+	"github.com/containers/podman/v5/cmd/podman/validate"
+	"github.com/containers/podman/v5/libpod/define"
 	"github.com/spf13/cobra"
 )
 
@@ -41,24 +38,12 @@ func init() {
 		Parent:  systemCmd,
 	})
 }
+
 func renumber(cmd *cobra.Command, args []string) {
-	// Shutdown all running engines, `renumber` will hijack all methods
-	registry.ContainerEngine().Shutdown(registry.Context())
-	registry.ImageEngine().Shutdown(registry.Context())
-
-	engine, err := infra.NewSystemEngine(entities.RenumberMode, registry.PodmanConfig())
-	if err != nil {
-		fmt.Println(err)
-		os.Exit(define.ExecErrorCodeGeneric)
-	}
-	defer engine.Shutdown(registry.Context())
-
-	err = engine.Renumber(registry.Context(), cmd.Flags(), registry.PodmanConfig())
-	if err != nil {
+	if err := registry.ContainerEngine().Renumber(registry.Context()); err != nil {
 		fmt.Println(err)
 		// FIXME change this to return the error like other commands
 		// defer will never run on os.Exit()
-		//nolint:gocritic
 		os.Exit(define.ExecErrorCodeGeneric)
 	}
 	os.Exit(0)
