@@ -737,6 +737,31 @@ RUN [[ -L /test/dummy-symlink ]] && echo SYMLNKOK || echo SYMLNKERR`, CITEST_IMA
 		Expect(session.OutputToString()).To(ContainSubstring("SYMLNKOK"))
 	})
 
+	It("podman build --no-hosts", func() {
+		targetPath := podmanTest.TempDir
+
+		containerFile := filepath.Join(targetPath, "Containerfile")
+		content := `FROM scratch
+RUN echo '56.78.12.34 image.example.com' > /etc/hosts`
+
+		Expect(os.WriteFile(containerFile, []byte(content), 0755)).To(Succeed())
+
+		defer func() {
+			Expect(os.RemoveAll(containerFile)).To(Succeed())
+		}()
+
+		session := podmanTest.Podman([]string{
+			"build", "--pull-never", "-t", "hosts_test", "--no-hosts", "--from", CITEST_IMAGE, targetPath,
+		})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(ExitCleanly())
+
+		session = podmanTest.Podman([]string{"run", "--no-hosts", "--rm", "hosts_test", "cat", "/etc/hosts"})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(ExitCleanly())
+		Expect(session.OutputToString()).To(Equal("56.78.12.34 image.example.com"))
+	})
+
 	It("podman build --from, --add-host, --cap-drop, --cap-add", func() {
 		targetPath := podmanTest.TempDir
 
@@ -947,4 +972,392 @@ RUN ls /dev/test1`, CITEST_IMAGE)
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(ExitWithError(1, `building at STEP "RUN --mount=type=cache,target=/test,z cat /test/world": while running runtime: exit status 1`))
 	})
+
+	It("podman build --build-context: local source", func() {
+		podmanTest.RestartRemoteService()
+
+		localCtx1 := filepath.Join(podmanTest.TempDir, "context1")
+		localCtx2 := filepath.Join(podmanTest.TempDir, "context2")
+
+		Expect(os.MkdirAll(localCtx1, 0755)).To(Succeed())
+		Expect(os.WriteFile(filepath.Join(localCtx1, "file1.txt"), []byte("Content from context1"), 0644)).To(Succeed())
+		Expect(os.WriteFile(filepath.Join(localCtx1, "config.json"), []byte(`{"source": "context1"}`), 0644)).To(Succeed())
+
+		Expect(os.MkdirAll(filepath.Join(localCtx2, "subdir"), 0755)).To(Succeed())
+		Expect(os.WriteFile(filepath.Join(localCtx2, "file2.txt"), []byte("Content from context2"), 0644)).To(Succeed())
+		Expect(os.WriteFile(filepath.Join(localCtx2, "subdir", "nested.txt"), []byte("Nested content"), 0644)).To(Succeed())
+
+		containerfile := `FROM quay.io/libpod/alpine:latest
+COPY --from=localctx1 /file1.txt /from-context1.txt
+COPY --from=localctx1 /config.json /config1.json`
+
+		containerfilePath := filepath.Join(podmanTest.TempDir, "Containerfile1")
+		Expect(os.WriteFile(containerfilePath, []byte(containerfile), 0644)).To(Succeed())
+
+		session := podmanTest.Podman([]string{
+			"build", "--pull-never", "-t", "test-local-single",
+			"--build-context", fmt.Sprintf("localctx1=%s", localCtx1),
+			"-f", containerfilePath, podmanTest.TempDir,
+		})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(ExitCleanly())
+
+		session = podmanTest.Podman([]string{"run", "--rm", "test-local-single", "cat", "/from-context1.txt"})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(ExitCleanly())
+		Expect(session.OutputToString()).To(Equal("Content from context1"))
+
+		containerfile = `FROM quay.io/libpod/alpine:latest
+COPY --from=ctx1 /file1.txt /file1.txt
+COPY --from=ctx2 /file2.txt /file2.txt
+COPY --from=ctx2 /subdir/nested.txt /nested.txt`
+
+		containerfilePath = filepath.Join(podmanTest.TempDir, "Containerfile2")
+		Expect(os.WriteFile(containerfilePath, []byte(containerfile), 0644)).To(Succeed())
+
+		session = podmanTest.Podman([]string{
+			"build", "--pull-never", "-t", "test-local-multi",
+			"--build-context", fmt.Sprintf("ctx1=%s", localCtx1),
+			"--build-context", fmt.Sprintf("ctx2=%s", localCtx2),
+			"-f", containerfilePath, podmanTest.TempDir,
+		})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(ExitCleanly())
+
+		session = podmanTest.Podman([]string{"run", "--rm", "test-local-multi", "cat", "/nested.txt"})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(ExitCleanly())
+		Expect(session.OutputToString()).To(Equal("Nested content"))
+
+		mainFile := filepath.Join(podmanTest.TempDir, "main.txt")
+		Expect(os.WriteFile(mainFile, []byte("From main context"), 0644)).To(Succeed())
+
+		containerfile = `FROM quay.io/libpod/alpine:latest
+COPY main.txt /main.txt
+COPY --from=additional /file1.txt /additional.txt`
+
+		containerfilePath = filepath.Join(podmanTest.TempDir, "Containerfile3")
+		Expect(os.WriteFile(containerfilePath, []byte(containerfile), 0644)).To(Succeed())
+
+		session = podmanTest.Podman([]string{
+			"build", "--pull-never", "-t", "test-local-mixed",
+			"--build-context", fmt.Sprintf("additional=%s", localCtx1),
+			"-f", containerfilePath, podmanTest.TempDir,
+		})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(ExitCleanly())
+
+		session = podmanTest.Podman([]string{"run", "--rm", "test-local-mixed", "cat", "/main.txt"})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(ExitCleanly())
+		Expect(session.OutputToString()).To(Equal("From main context"))
+
+		session = podmanTest.Podman([]string{"rmi", "-f", "test-local-single", "test-local-multi", "test-local-mixed"})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(ExitCleanly())
+	})
+
+	It("podman build --build-context: URL source", func() {
+		podmanTest.RestartRemoteService()
+
+		testRepoURL := "https://github.com/containers/PodmanHello.git"
+		testArchiveURL := "https://github.com/containers/PodmanHello/archive/refs/heads/main.tar.gz"
+
+		containerfile := `FROM quay.io/libpod/alpine:latest
+COPY --from=urlctx . /url-context/`
+
+		containerfilePath := filepath.Join(podmanTest.TempDir, "ContainerfileURL1")
+		Expect(os.WriteFile(containerfilePath, []byte(containerfile), 0644)).To(Succeed())
+
+		session := podmanTest.Podman([]string{
+			"build", "--pull-never", "-t", "test-url-single",
+			"--build-context", fmt.Sprintf("urlctx=%s", testRepoURL),
+			"-f", containerfilePath, podmanTest.TempDir,
+		})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(ExitCleanly())
+
+		session = podmanTest.Podman([]string{"run", "--rm", "test-url-single", "ls", "/url-context/"})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(ExitCleanly())
+		output := session.OutputToString()
+		Expect(output).To(ContainSubstring("LICENSE"))
+		Expect(output).To(ContainSubstring("README.md"))
+
+		containerfile = `FROM quay.io/libpod/alpine:latest
+COPY --from=archive . /from-archive/`
+
+		containerfilePath = filepath.Join(podmanTest.TempDir, "ContainerfileURL2")
+		Expect(os.WriteFile(containerfilePath, []byte(containerfile), 0644)).To(Succeed())
+
+		session = podmanTest.Podman([]string{
+			"build", "--pull-never", "-t", "test-archive",
+			"--build-context", fmt.Sprintf("archive=%s", testArchiveURL),
+			"-f", containerfilePath, podmanTest.TempDir,
+		})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(ExitCleanly())
+
+		session = podmanTest.Podman([]string{"run", "--rm", "test-archive", "ls", "/from-archive/"})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(ExitCleanly())
+		output = session.OutputToString()
+		Expect(output).To(ContainSubstring("PodmanHello-main"))
+
+		session = podmanTest.Podman([]string{"run", "--rm", "test-archive", "ls", "/from-archive/PodmanHello-main/"})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(ExitCleanly())
+		output = session.OutputToString()
+		Expect(output).To(ContainSubstring("LICENSE"))
+		Expect(output).To(ContainSubstring("README.md"))
+
+		localCtx := filepath.Join(podmanTest.TempDir, "localcontext")
+		Expect(os.MkdirAll(localCtx, 0755)).To(Succeed())
+		Expect(os.WriteFile(filepath.Join(localCtx, "local.txt"), []byte("Local content"), 0644)).To(Succeed())
+
+		containerfile = `FROM quay.io/libpod/alpine:latest
+COPY --from=urlrepo . /from-url/
+COPY --from=localctx /local.txt /local.txt
+RUN echo "Combined URL and local contexts" > /combined.txt`
+
+		containerfilePath = filepath.Join(podmanTest.TempDir, "ContainerfileURL3")
+		Expect(os.WriteFile(containerfilePath, []byte(containerfile), 0644)).To(Succeed())
+
+		session = podmanTest.Podman([]string{
+			"build", "--pull-never", "-t", "test-url-mixed",
+			"--build-context", fmt.Sprintf("urlrepo=%s", testRepoURL),
+			"--build-context", fmt.Sprintf("localctx=%s", localCtx),
+			"-f", containerfilePath, podmanTest.TempDir,
+		})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(ExitCleanly())
+
+		session = podmanTest.Podman([]string{"run", "--rm", "test-url-mixed", "cat", "/local.txt"})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(ExitCleanly())
+		Expect(session.OutputToString()).To(Equal("Local content"))
+
+		mainFile := filepath.Join(podmanTest.TempDir, "main-url-test.txt")
+		Expect(os.WriteFile(mainFile, []byte("Main context for URL test"), 0644)).To(Succeed())
+
+		containerfile = `FROM quay.io/libpod/alpine:latest
+COPY main-url-test.txt /main.txt
+COPY --from=gitrepo . /git-repo/`
+
+		containerfilePath = filepath.Join(podmanTest.TempDir, "ContainerfileURL5")
+		Expect(os.WriteFile(containerfilePath, []byte(containerfile), 0644)).To(Succeed())
+
+		session = podmanTest.Podman([]string{
+			"build", "--pull-never", "-t", "test-url-main",
+			"--build-context", fmt.Sprintf("gitrepo=%s", testRepoURL),
+			"-f", containerfilePath, podmanTest.TempDir,
+		})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(ExitCleanly())
+
+		session = podmanTest.Podman([]string{"run", "--rm", "test-url-main", "cat", "/main.txt"})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(ExitCleanly())
+		Expect(session.OutputToString()).To(Equal("Main context for URL test"))
+
+		session = podmanTest.Podman([]string{"rmi", "-f", "test-url-single", "test-archive", "test-url-mixed", "test-url-main"})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(ExitCleanly())
+	})
+
+	It("podman build --build-context: Image source", func() {
+		podmanTest.RestartRemoteService()
+
+		alpineImage := "quay.io/libpod/alpine:latest"
+		busyboxImage := "quay.io/libpod/busybox:latest"
+
+		containerfile := `FROM quay.io/libpod/busybox:latest AS source
+FROM quay.io/libpod/alpine:latest
+COPY --from=source /bin/busybox /busybox-from-stage`
+
+		containerfilePath := filepath.Join(podmanTest.TempDir, "ContainerfileMultiStage")
+		Expect(os.WriteFile(containerfilePath, []byte(containerfile), 0644)).To(Succeed())
+
+		session := podmanTest.Podman([]string{
+			"build", "--pull-never", "-t", "test-multi-stage",
+			"-f", containerfilePath, podmanTest.TempDir,
+		})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(ExitCleanly())
+
+		testCases := []struct {
+			name          string
+			prefix        string
+			image         string
+			contextName   string
+			containerfile string
+			verifyCmd     []string
+		}{
+			{
+				name:        "docker-image-prefix",
+				prefix:      "docker-image://",
+				image:       alpineImage,
+				contextName: "dockerimg",
+				containerfile: `FROM quay.io/libpod/alpine:latest
+COPY --from=dockerimg /etc/alpine-release /alpine-version.txt`,
+				verifyCmd: []string{"cat", "/alpine-version.txt"},
+			},
+			{
+				name:        "container-image-prefix",
+				prefix:      "container-image://",
+				image:       busyboxImage,
+				contextName: "containerimg",
+				containerfile: `FROM quay.io/libpod/alpine:latest
+COPY --from=containerimg /bin/busybox /busybox-binary`,
+				verifyCmd: []string{"/busybox-binary", "--help"},
+			},
+			{
+				name:        "docker-prefix",
+				prefix:      "docker://",
+				image:       alpineImage,
+				contextName: "dockershort",
+				containerfile: `FROM quay.io/libpod/alpine:latest
+COPY --from=dockershort /etc/os-release /os-release.txt`,
+				verifyCmd: []string{"cat", "/os-release.txt"},
+			},
+		}
+
+		for _, tc := range testCases {
+			containerfilePath = filepath.Join(podmanTest.TempDir, fmt.Sprintf("Containerfile_%s", tc.name))
+			Expect(os.WriteFile(containerfilePath, []byte(tc.containerfile), 0644)).To(Succeed())
+
+			session = podmanTest.Podman([]string{
+				"build", "--pull-never", "-t", fmt.Sprintf("test-%s", tc.name),
+				"--build-context", fmt.Sprintf("%s=%s%s", tc.contextName, tc.prefix, tc.image),
+				"-f", containerfilePath, podmanTest.TempDir,
+			})
+			session.WaitWithDefaultTimeout()
+			Expect(session).Should(ExitCleanly())
+
+			session = podmanTest.Podman(append([]string{"run", "--rm", fmt.Sprintf("test-%s", tc.name)}, tc.verifyCmd...))
+			session.WaitWithDefaultTimeout()
+			Expect(session).Should(ExitCleanly())
+			if tc.name == "container-image-prefix" {
+				Expect(session.OutputToString()).To(ContainSubstring("BusyBox"))
+			}
+		}
+
+		session = podmanTest.Podman([]string{"rmi", "-f", "test-multi-stage"})
+		session.WaitWithDefaultTimeout()
+		for _, tc := range testCases {
+			session = podmanTest.Podman([]string{"rmi", "-f", fmt.Sprintf("test-%s", tc.name)})
+			session.WaitWithDefaultTimeout()
+		}
+	})
+
+	It("podman build --build-context: Mixed source", func() {
+		podmanTest.RestartRemoteService()
+
+		localCtx := filepath.Join(podmanTest.TempDir, "local-mixed")
+		Expect(os.MkdirAll(localCtx, 0755)).To(Succeed())
+		Expect(os.WriteFile(filepath.Join(localCtx, "local-config.json"), []byte(`{"context": "local", "version": "1.0"}`), 0644)).To(Succeed())
+		Expect(os.WriteFile(filepath.Join(localCtx, "app.conf"), []byte("# Local app configuration\nmode=production\nport=8080"), 0644)).To(Succeed())
+
+		urlContext := "https://github.com/containers/PodmanHello.git"
+		alpineImage := "quay.io/libpod/alpine:latest"
+		busyboxImage := "quay.io/libpod/busybox:latest"
+
+		mainFile := filepath.Join(podmanTest.TempDir, "VERSION")
+		Expect(os.WriteFile(mainFile, []byte("v1.0.0-mixed"), 0644)).To(Succeed())
+
+		containerfile := `FROM quay.io/libpod/alpine:latest
+
+# From main build context
+COPY VERSION /app/VERSION
+
+# From local directory context
+COPY --from=localdir /local-config.json /app/config/local-config.json
+COPY --from=localdir /app.conf /app/config/app.conf
+
+# From URL/Git context
+COPY --from=gitrepo /LICENSE /app/licenses/podman-hello-LICENSE
+COPY --from=gitrepo /README.md /app/docs/podman-hello-README.md
+
+# From image contexts with different prefixes
+COPY --from=alpineimg /etc/alpine-release /app/base-images/alpine-version
+COPY --from=busyboximg /bin/busybox /app/tools/busybox
+
+# Create a summary file
+RUN echo "Build with all context types completed" > /app/build-summary.txt && \
+    chmod +x /app/tools/busybox
+
+WORKDIR /app`
+
+		containerfilePath := filepath.Join(podmanTest.TempDir, "ContainerfileMixed")
+		Expect(os.WriteFile(containerfilePath, []byte(containerfile), 0644)).To(Succeed())
+
+		session := podmanTest.Podman([]string{
+			"build", "--pull-never", "-t", "test-all-contexts",
+			"--build-context", fmt.Sprintf("localdir=%s", localCtx),
+			"--build-context", fmt.Sprintf("gitrepo=%s", urlContext),
+			"--build-context", fmt.Sprintf("alpineimg=docker-image://%s", alpineImage),
+			"--build-context", fmt.Sprintf("busyboximg=container-image://%s", busyboxImage),
+			"-f", containerfilePath, podmanTest.TempDir,
+		})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(ExitCleanly())
+
+		verifyTests := []struct {
+			cmd      []string
+			expected string
+		}{
+			{[]string{"cat", "/app/VERSION"}, "v1.0.0-mixed"},
+			{[]string{"cat", "/app/config/local-config.json"}, `"context": "local"`},
+			{[]string{"cat", "/app/config/app.conf"}, "port=8080"},
+			{[]string{"test", "-f", "/app/licenses/podman-hello-LICENSE"}, ""},
+			{[]string{"test", "-f", "/app/docs/podman-hello-README.md"}, ""},
+			{[]string{"cat", "/app/base-images/alpine-version"}, "3."},
+			{[]string{"/app/tools/busybox", "--help"}, "BusyBox"},
+			{[]string{"cat", "/app/build-summary.txt"}, "Build with all context types completed"},
+		}
+
+		for _, test := range verifyTests {
+			session = podmanTest.Podman(append([]string{"run", "--rm", "test-all-contexts"}, test.cmd...))
+			session.WaitWithDefaultTimeout()
+			Expect(session).Should(ExitCleanly())
+			if test.expected != "" {
+				Expect(session.OutputToString()).To(ContainSubstring(test.expected))
+			}
+		}
+
+		session = podmanTest.Podman([]string{
+			"run", "--rm", "test-all-contexts",
+			"/app/tools/busybox", "grep", "port", "/app/config/app.conf",
+		})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(ExitCleanly())
+		Expect(session.OutputToString()).To(ContainSubstring("port=8080"))
+
+		containerfile2 := `FROM quay.io/libpod/alpine:latest
+COPY --from=img1 /etc/os-release /prefix-test/docker-prefix.txt
+COPY --from=img2 /etc/alpine-release /prefix-test/container-prefix.txt`
+
+		containerfilePath2 := filepath.Join(podmanTest.TempDir, "ContainerfileMixed2")
+		Expect(os.WriteFile(containerfilePath2, []byte(containerfile2), 0644)).To(Succeed())
+
+		session = podmanTest.Podman([]string{
+			"build", "--pull-never", "-t", "test-prefix-mix",
+			"--build-context", fmt.Sprintf("img1=docker://%s", alpineImage),
+			"--build-context", fmt.Sprintf("img2=container-image://%s", alpineImage),
+			"-f", containerfilePath2, podmanTest.TempDir,
+		})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(ExitCleanly())
+
+		session = podmanTest.Podman([]string{"run", "--rm", "test-prefix-mix", "ls", "/prefix-test/"})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(ExitCleanly())
+		output := session.OutputToString()
+		Expect(output).To(ContainSubstring("docker-prefix.txt"))
+		Expect(output).To(ContainSubstring("container-prefix.txt"))
+
+		session = podmanTest.Podman([]string{"rmi", "-f", "test-all-contexts", "test-prefix-mix"})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(ExitCleanly())
+	})
 })
