@@ -1,3 +1,5 @@
+//go:build linux || freebsd
+
 package integration
 
 import (
@@ -8,7 +10,6 @@ import (
 	"github.com/containers/storage/pkg/stringid"
 	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
-	. "github.com/onsi/gomega/gexec"
 )
 
 var _ = Describe("Podman diff", func() {
@@ -23,7 +24,7 @@ var _ = Describe("Podman diff", func() {
 	It("podman diff bogus image", func() {
 		session := podmanTest.Podman([]string{"diff", "1234"})
 		session.WaitWithDefaultTimeout()
-		Expect(session).Should(Exit(125))
+		Expect(session).Should(ExitWithError(125, "1234 not found: layer not known"))
 	})
 
 	It("podman diff image with json output", func() {
@@ -103,7 +104,7 @@ RUN echo test
 		session = podmanTest.Podman([]string{"image", "diff", image, baseImage})
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(ExitCleanly())
-		Expect(session.OutputToStringArray()).To(HaveLen(4))
+		Expect(session.OutputToStringArray()).To(HaveLen(3))
 		Expect(session.OutputToString()).To(ContainSubstring("A " + file1))
 		Expect(session.OutputToString()).To(ContainSubstring("A " + file2))
 		Expect(session.OutputToString()).To(ContainSubstring("A " + file3))
@@ -119,7 +120,7 @@ RUN echo test
 	It("podman image diff bogus image", func() {
 		session := podmanTest.Podman([]string{"image", "diff", "1234", ALPINE})
 		session.WaitWithDefaultTimeout()
-		Expect(session).Should(Exit(125))
+		Expect(session).Should(ExitWithError(125, "1234 not found: 1234: image not known"))
 	})
 
 	It("podman image diff of the same image", func() {
@@ -135,8 +136,8 @@ RUN echo test
 
 		// Create container image with the files
 		containerfile := fmt.Sprintf(`
-FROM  %s
-RUN touch %s`, ALPINE, imagefile)
+	 FROM  %s
+	 RUN touch %s`, ALPINE, imagefile)
 
 		name := "podman-diff-test"
 		podmanTest.BuildImage(containerfile, name, "false")
@@ -149,13 +150,13 @@ RUN touch %s`, ALPINE, imagefile)
 		session = podmanTest.Podman([]string{"diff", name})
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(ExitCleanly())
-		Expect(session.OutputToStringArray()).To(HaveLen(2))
+		Expect(session.OutputToStringArray()).To(HaveLen(1))
 		Expect(session.OutputToString()).To(ContainSubstring(imagefile))
 
 		session = podmanTest.Podman([]string{"image", "diff", name})
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(ExitCleanly())
-		Expect(session.OutputToStringArray()).To(HaveLen(2))
+		Expect(session.OutputToStringArray()).To(HaveLen(1))
 		Expect(session.OutputToString()).To(ContainSubstring(imagefile))
 
 		// container diff has to show the container
@@ -166,4 +167,13 @@ RUN touch %s`, ALPINE, imagefile)
 		Expect(session.OutputToString()).To(ContainSubstring(confile))
 	})
 
+	It("podman diff without args", func() {
+		session := podmanTest.Podman([]string{"diff"})
+		session.WaitWithDefaultTimeout()
+		if IsRemote() {
+			Expect(session).Should(ExitWithError(125, " requires a name or id"))
+		} else {
+			Expect(session).Should(ExitWithError(125, " requires a name, id, or the \"--latest\" flag"))
+		}
+	})
 })
